package com.ym.mec.web.controller;

import com.ym.mec.biz.dal.wrapper.MemberRankSettingWrapper;
import com.ym.mec.biz.service.MemberRankSettingService;
import com.ym.mec.biz.service.SysUserService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.yonge.log.model.AuditLogAnnotation;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.HashMap;
import java.util.List;

@RequestMapping("${app-config.url.web:}/memberRankSetting")
@Api(tags = "会员等级服务")
@RestController
public class MemberRankSettingController extends BaseController {

    @Autowired
    private MemberRankSettingService memberRankSettingService;
    @Autowired
    private SysUserService sysUserService;

    @ApiOperation(value = "新增")
    @PostMapping("/add")
    @PreAuthorize("@pcs.hasPermissions('memberRankSetting/add')")
    @AuditLogAnnotation(operateName = "新增会员等级")
    public HttpResponseResult<Object> add(@RequestBody MemberRankSettingWrapper.MemberRankSettingAdd memberRankSetting) {
        memberRankSetting.setOperator(sysUserService.getUserId());
        memberRankSetting.setUpdateTime(new Date());
        memberRankSettingService.add(memberRankSetting);
        return succeed();
    }

    @ApiOperation(value = "获取详情")
    @GetMapping("/get")
    @PreAuthorize("@pcs.hasPermissions('memberRankSetting/get')")
    @AuditLogAnnotation(operateName = "会员等级详情")
    public HttpResponseResult<MemberRankSettingWrapper.MemberRankSetting> get(Integer memberRankSettingId) {
        return succeed(memberRankSettingService.getDetail(memberRankSettingId));
    }

    @ApiOperation(value = "修改")
    @PostMapping("/update")
    @PreAuthorize("@pcs.hasPermissions('memberRankSetting/update')")
    @AuditLogAnnotation(operateName = "修改会员等级")
    public HttpResponseResult<Object> update(@RequestBody MemberRankSettingWrapper.MemberRankSettingAdd memberRankSetting) {
        memberRankSetting.setOperator(sysUserService.getUserId());
        memberRankSettingService.updateSetting(memberRankSetting);
        return succeed();
    }



    @ApiOperation(value = "删除")
    @PostMapping("/del/{id}")
    @PreAuthorize("@pcs.hasPermissions('memberRankSetting/del')")
    @AuditLogAnnotation(operateName = "删除学练宝会员等级")
    public HttpResponseResult<Object> update(@PathVariable Integer id) {
        memberRankSettingService.del(id);
        return succeed();
    }
    @ApiOperation(value = "分页查询")
    @PostMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('memberRankSetting/queryPage')")
    public HttpResponseResult<List<MemberRankSettingWrapper.MemberRankSetting>> queryPage(@RequestBody MemberRankSettingWrapper.MemberRankSettingQuery queryInfo) {
        return succeed(memberRankSettingService.queryPage2(queryInfo));
    }

    @ApiOperation(value = "树状列表")
    @GetMapping("/categoryTree")
    public HttpResponseResult<MemberRankSettingWrapper.Category> queryTree(@RequestParam(required = false) Integer parentMemberId) {

        return succeed(memberRankSettingService.getTree(parentMemberId));
    }

    @ApiOperation(value = "获取所有会员列表")
    @GetMapping("/findAll")
    @PreAuthorize("@pcs.hasPermissions('memberRankSetting/findAll')")
    public Object findAll(Boolean isDefault) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("isDefault",isDefault);
        return succeed(memberRankSettingService.findAll(map));
    }

    @ApiOperation(value = "获取可用的会员树状列表")
    @GetMapping("/queryTree")
    public HttpResponseResult<List<MemberRankSettingWrapper.MemberRankSetting>> queryTree(String musicGroupId,Integer organId,Integer studentId) {
        if (musicGroupId == null && organId == null && studentId == null) {
            return failed("参数错误");
        }
        return succeed(memberRankSettingService.queryTree(musicGroupId,organId,studentId));
    }
}
