package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ym.mec.biz.dal.dao.StudentLessonExaminationDetailDao;
import com.ym.mec.biz.dal.dto.LessonExaminationDetailDto;
import com.ym.mec.biz.dal.dto.StudentLessonExaminationDetailDto;
import com.ym.mec.biz.dal.dto.StudentLessonExaminationSaveDto;
import com.ym.mec.biz.dal.entity.StudentLessonExaminationDetail;
import com.ym.mec.biz.dal.page.StudentLessonExaminationQueryDto;
import com.ym.mec.biz.service.StudentLessonExaminationDetailService;
import com.ym.mec.common.exception.BizException;
import org.springframework.stereotype.Service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 进度评测详情表(StudentLessonExaminationDetail)表服务实现类
 *
 * @author zx
 * @since 2023-04-03 18:22:01
 */
@Service("studentLessonExaminationDetailService")
public class StudentLessonExaminationDetailServiceImpl extends ServiceImpl<StudentLessonExaminationDetailDao, StudentLessonExaminationDetail> implements StudentLessonExaminationDetailService {

    private final static Logger log = LoggerFactory.getLogger(StudentLessonExaminationDetailServiceImpl.class);

    @Override
    public StudentLessonExaminationDetailDao getDao() {
        return this.baseMapper;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void save(Long lessonExaminationId,List<StudentLessonExaminationSaveDto> studentLessonExaminationSaveDtos) {
        List<StudentLessonExaminationDetail> detailList = new ArrayList<>();
        for (StudentLessonExaminationSaveDto dto : studentLessonExaminationSaveDtos) {
            String[] studentIds = dto.getStudentIds().split(",");
            List<LessonExaminationDetailDto> detailDtos = dto.getExaminationDetailDtos();
            if (CollectionUtils.isEmpty(detailDtos) && StringUtils.isEmpty(dto.getMemo())) {
                throw new BizException("有部分声部未设置作业内容");
            }
            if (CollectionUtils.isEmpty(detailDtos)) {
//                throw new BizException("有部分声部未选择曲目");
                for (String id : studentIds) {
                    Integer studentId = Integer.parseInt(id);
                    for (int i = 0; i < 4; i++) {
                        StudentLessonExaminationDetail studentLessonExaminationDetail = new StudentLessonExaminationDetail();
                        studentLessonExaminationDetail.setUserId(studentId);
                        studentLessonExaminationDetail.setMemo(dto.getMemo());
                        studentLessonExaminationDetail.setLessonExaminationId(lessonExaminationId);
                        studentLessonExaminationDetail.setGroup(dto.getGroup());
                        studentLessonExaminationDetail.setType("DESCRIPTION");
                        detailList.add(studentLessonExaminationDetail);
                    }
                }
            } else {
                for (String id : studentIds) {
                    Integer studentId = Integer.parseInt(id);
                    Integer score = 100 / detailDtos.size();
                    Integer subScore = 100 - score * detailDtos.size();
                    for (int i = 0; i < detailDtos.size(); i++) {
                        LessonExaminationDetailDto vo = detailDtos.get(i);
                        StudentLessonExaminationDetail studentLessonExaminationDetail = new StudentLessonExaminationDetail();
                        studentLessonExaminationDetail.setUserId(studentId);
                        studentLessonExaminationDetail.setMemo(dto.getMemo());
                        studentLessonExaminationDetail.setLessonExaminationId(lessonExaminationId);
                        studentLessonExaminationDetail.setMusicScoreId(vo.getMusicScoreId());
                        studentLessonExaminationDetail.setHeardLevel(vo.getHeardLevel());
                        studentLessonExaminationDetail.setStandardScore(vo.getScore());
                        studentLessonExaminationDetail.setStart(vo.getStart());
                        studentLessonExaminationDetail.setType("MUSIC_SCORE");
                        studentLessonExaminationDetail.setEnd(vo.getEnd());
                        studentLessonExaminationDetail.setGroup(dto.getGroup());
                        if (i == 0) {
                            studentLessonExaminationDetail.setAvgScore(score + subScore);
                        } else {
                            studentLessonExaminationDetail.setAvgScore(score);
                        }
                        studentLessonExaminationDetail.setPartIndex(vo.getPartIndex());
                        detailList.add(studentLessonExaminationDetail);
                    }
                }
            }
        }
        baseMapper.insertBatch(detailList);
    }

    @Override
    public List<StudentLessonExaminationDetailDto> queryAll(StudentLessonExaminationQueryDto query) {
        return baseMapper.queryAll(query);
    }

    @Override
    public List<StudentLessonExaminationDetail> queryByStudent(Integer userId, Long lessonExaminationId) {
        return this.lambdaQuery()
            .eq(StudentLessonExaminationDetail::getUserId, userId)
            .eq(StudentLessonExaminationDetail::getLessonExaminationId, lessonExaminationId)
            .list();
    }

}

